import matplotlib.pyplot as plt
import math

def plotDistributionComparison(histograms, legend, title):
    '''
    Plots a list of histograms with matching list of descriptions as the legend
    '''
    # determine max. length
    max_length = max(len(x) for x in histograms)
    
    # extend "shorter" distributions
    for x in histograms:
        x.extend([0.0]* (max_length-len(x)) )
        
    # plots histograms
    for h in histograms:
        plt.plot(range(len(h)), h, marker = 'x')
    
    # remember: never forget labels!
    plt.xlabel('degree')
    plt.ylabel('P')
    
    # you don't have to do something stuff here
    plt.legend(legend)
    plt.title(title)
    plt.tight_layout()
    plt.show()

def plotDistributionComparisonLogLog(histograms, legend, title):
    '''
    Plots a list of histograms with matching list of descriptions as the legend
    '''
    # determine max. length
    max_length = max(len(x) for x in histograms)
    
    # extend "shorter" distributions
    for x in histograms:
        x.extend([0.0]* (max_length-len(x)) )
         
    # plots histograms
    for h in histograms:
        plt.plot([math.log(x) if x > 0 else 0.0 for x in range(len(h))], [math.log(x) if x > 0 else 0.0 for x in h], marker = 'x', linestyle='')
    
    # remember: never forget labels!
    plt.xlabel('log degree')
    plt.ylabel('log P')
    
    # you don't have to do something stuff here
    plt.legend(legend)
    plt.title(title)
    plt.tight_layout()
    plt.show()
    
def fac(x):
    '''
    Compute factorial
    '''
    if x==0:
        return 1
    else:
        return x* fac(x-1)
  
def poisson(k, lamb):
    '''
    Compute Poisson distribution P(k)
    '''
    return( (lamb**k)/fac(k) * math.exp(-lamb) )

def getPoissonDistributionHistogram(num_nodes, num_links, k):
    '''
    Generates a Poisson distribution histogram up to k
    '''
    # according to lecture 1
    lamb = num_links / ((num_nodes-1)/2.0)
    
    histogram = (k+1)  * [0]
    for i in range(k):
        histogram[i] = poisson(i, lamb)
        
    return histogram

def getScaleFreeDistributionHistogram(gamma, k):
    '''
    Generates a Power law distribution histogram with slope gamma up to degree k
    '''
    histogram = (k+1)  * [0]
    
    # generate
    for i in range(1, k):
        histogram[i] = math.pow(i, -gamma)
    
    # normalize
    C = 1 / float(sum(histogram))
    histogram = [C*x for x in histogram]
    
    return histogram

def simpleKSdist(histogram_a, histogram_b):
    '''
    Simple Kolmogorov-Smirnov distance implementation
    '''
    # for simplicity (and our case), assume histograms of equal length
    assert(len(histogram_a) == len(histogram_b))
    
    n = len(histogram_a)
    C_a = n * [0.0]
    C_b = n * [0.0]
    
    # convert PDFs to CDFs
    C_a[0] = histogram_a[0]
    C_b[0] = histogram_b[0] 
    for i in range(2, n):
        C_a[i] = C_a[i-1] + histogram_a[i]
        C_b[i] = C_b[i-1] + histogram_b[i]
        
    # distances
    D = n * [0.0]
    for i in range(1, n):
        D[i] = abs(C_a[i] - C_b[i])
    
    return max(D) # return D_max
    
#### forces and layout from here

def plotEnergies(energies, legend, title):
    '''
    Plots a list of energies from layouting
    '''
    for e in energies:
        plt.plot(e)
    
    # remember: never forget labels!
    plt.xlabel('iteration')
    plt.ylabel('E')
    
    # you don't have to do something stuff here
    plt.legend(legend)
    plt.title(title)
    plt.tight_layout()
    plt.show()


def plotLayout(network, title):
    '''
    Plots layout of a network
    '''
    nodelist = list(network.nodes.values())
    
    # plot connecting lines
    for node1 in nodelist:
        for node2 in node1.nodelist:
            if node1 < node2: continue
            plt.plot([node1.pos_x, node2.pos_x], [node1.pos_y, node2.pos_y], linestyle = '-', color = 'black')
    
    # plot points
    for node in nodelist:
        x, y = node.pos_x, node.pos_y
        plt.plot(x, y, marker='H', color = 'red')
            
    plt.title(title)
    plt.tight_layout()
    plt.show()